"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.uploadFiles = exports.fetch = exports.setStorage = exports.getStorage = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _index = require("../npm/@tarojs/taro-weapp/index.js");

var _index2 = _interopRequireDefault(_index);

var _api = require("../constants/api.js");

var _login = require("./login.js");

var _login2 = _interopRequireDefault(_login);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var CODE_SUCCESS = 1000;
var CODE_UNLOGIN = 1001;
var CODE_AUTH_EXPIRED = 1002;
var accountInfo = wx.getAccountInfoSync();
var appId = accountInfo.miniProgram.appId;

var getStorage = exports.getStorage = function getStorage(key) {
  return _index2.default.getStorage({ key: key }).then(function (res) {
    return res.data;
  }).catch(function () {
    return '';
  });
};

var setStorage = exports.setStorage = function setStorage(key, data) {
  return _index2.default.setStorage({ key: key, data: data });
};

// function updateToken(data = {}) {
//   return Promise.all([
//     Taro.setStorage({ key: 'token', data: data['token'] || '' })
//   ])
// }

/**
 * 简易封装网络请求
 * // NOTE 需要注意 RN 不支持 *StorageSync，此处用 async/await 解决
 * @param {*} options
 */
var fetch = exports.fetch = function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(options) {
    var url, payload, _options$method, method, _options$showToast, showToast, _options$autoLogin, autoLogin;

    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            url = options.url, payload = options.payload, _options$method = options.method, method = _options$method === undefined ? 'GET' : _options$method, _options$showToast = options.showToast, showToast = _options$showToast === undefined ? true : _options$showToast, _options$autoLogin = options.autoLogin, autoLogin = _options$autoLogin === undefined ? true : _options$autoLogin;
            return _context2.abrupt("return", _index2.default.request({
              url: url,
              method: method,
              data: payload,
              header: {
                'content-type': 'application/json',
                'authorization': "Bearer " + _index2.default.getStorageSync('token'),
                'appId': appId,
                'x-action': 'miniapp'
              }
            }).then(function () {
              var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(res) {
                var _res$data, data, code;

                return regeneratorRuntime.wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        _res$data = res.data, data = _res$data.data, code = _res$data.code;

                        if (!(code !== CODE_SUCCESS)) {
                          _context.next = 3;
                          break;
                        }

                        return _context.abrupt("return", Promise.reject(res.data));

                      case 3:
                        return _context.abrupt("return", data);

                      case 4:
                      case "end":
                        return _context.stop();
                    }
                  }
                }, _callee, undefined);
              }));

              return function (_x2) {
                return _ref2.apply(this, arguments);
              };
            }()).catch(function (err) {
              var isExpired = err.code === CODE_AUTH_EXPIRED || err.code === CODE_UNLOGIN;
              var defaultMsg = isExpired ? '登录失效' : err.message || '请求异常';
              if (isExpired && autoLogin) {
                // token失效，重新登录后，跳转到首页
                (0, _login2.default)({}, function () {
                  _index2.default.switchTab({
                    url: '/pages/project/index'
                  });
                });
              } else {
                if (showToast) {
                  _index2.default.showToast({
                    title: err && err.msg || defaultMsg,
                    icon: 'none'
                  });
                }
              }
              return Promise.reject(_extends({ message: defaultMsg }, err));
            }));

          case 2:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  }));

  return function fetch(_x) {
    return _ref.apply(this, arguments);
  };
}();

var uploadFiles = exports.uploadFiles = function () {
  var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(files) {
    var uploads, token, header, i;
    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            uploads = [];
            _context3.next = 3;
            return getStorage('token');

          case 3:
            token = _context3.sent;
            header = token ? { 'authorization': "Bearer " + token } : {};

            for (i = 0; i < files.length; i++) {
              uploads[i] = new Promise(function (resolve, reject) {
                wx.uploadFile({
                  url: _api.API_UPLOAD_IMAGE,
                  filePath: files[i],
                  header: header,
                  name: 'file',
                  success: function success(res) {
                    var _data = JSON.parse(res.data);
                    resolve(_data.data);
                  },
                  fail: function fail(err) {
                    reject(err);
                  }
                });
              });
            }

            return _context3.abrupt("return", Promise.all(uploads));

          case 7:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3, undefined);
  }));

  return function uploadFiles(_x3) {
    return _ref3.apply(this, arguments);
  };
}();