'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isEmail = exports.isChineseMobilePhoneNumber = exports.validateAuth = exports.deepClone = exports.isObject = exports.makeCardShareImg = exports.roundRect = exports.makeCardPoster = exports.downloadFile = exports.saveFile = exports.canvasToTempFilePath = exports.removeLocalFile = exports.makeProjectPoster = exports.getQueryString = exports.wait = exports.pageToLogin = exports.getCurrentPageUrl = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.getDownloadURL = getDownloadURL;
exports.throttle = throttle;

var _index = require('../npm/@tarojs/taro-weapp/index.js');

var _index2 = _interopRequireDefault(_index);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getDownloadURL(url, type) {
  switch (type) {
    case 'avatar':
      return url.replace('https://wx.qlogo.cn/', 'https://lt.pawoma.cn/qlogo/');
    case 'alioss':
      return url.replace('https://estateagents.oss-cn-shanghai.aliyuncs.com/', 'https://lt.pawoma.cn/alioss/');
    default:
      return url;
  }
}

/**
 * @description 获取当前页url
 */
var getCurrentPageUrl = exports.getCurrentPageUrl = function getCurrentPageUrl() {
  var pages = _index2.default.getCurrentPages();
  var currentPage = pages[pages.length - 1];
  var url = currentPage.route;
  return url;
};

var pageToLogin = exports.pageToLogin = function pageToLogin() {
  var path = getCurrentPageUrl();
  if (!path.includes('login')) {
    _index2.default.navigateTo({
      url: "/pages/login/login"
    });
  }
};

var endTime = void 0;
function throttle(callback) {
  var time = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 500;

  var startTime = new Date().getTime();
  if (startTime - endTime > time || !endTime) {
    callback();
    endTime = startTime;
  }
}

var wait = exports.wait = function wait() {
  var time = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 500;

  return new Promise(function (res) {
    setTimeout(function () {
      res();
    }, time);
  });
};

var getQueryString = exports.getQueryString = function getQueryString(str, name) {
  var regex = new RegExp("(^|&)" + name + "=([^&]*)(&|$)");
  var r = str.match(regex);
  if (r != null) return unescape(r[2]);
  return null;
};
// 项目详情海报
var makeProjectPoster = exports.makeProjectPoster = function makeProjectPoster(res) {
  return new Promise(function (resolve) {
    var poster = res.poster,
        qrcode = res.qrcode,
        nickname = res.nickname,
        avatarurl = res.avatarurl,
        canvasName = res.canvasName;

    var ctx = _index2.default.createCanvasContext(canvasName);
    var _avatarurl = getDownloadURL(avatarurl, 'avatar');
    var _poster = getDownloadURL(poster, 'alioss');

    Promise.all([_index2.default.downloadFile({ url: _poster }), _index2.default.downloadFile({ url: qrcode }), _index2.default.downloadFile({ url: _avatarurl })]).then(function (list) {

      ctx.setFillStyle("#ffffff");
      ctx.fillRect(0, 0, 640, 1136);

      // 背景海报
      ctx.drawImage(list[0].tempFilePath, 0, 0, 640, 1136);

      //绘制名字
      ctx.setFontSize(30);
      ctx.setFillStyle('#fff');
      ctx.setTextAlign('left');
      ctx.fillText(nickname, 160, 920, 250);
      ctx.stroke();
      ctx.save();
      ctx.arc(500, 910, 60, 0, 2 * Math.PI);
      ctx.strokeStyle = "#ffffff";
      ctx.setFillStyle('#fff');
      ctx.fill();
      ctx.clip(); //裁剪上面的圆形
      ctx.drawImage(list[1].tempFilePath, 0, 0, 430, 430, 440, 850, 120, 120);
      ctx.restore();

      //绘制头像
      ctx.arc(96, 906, 50, 0, 2 * Math.PI); //画出圆
      ctx.strokeStyle = "#ffffff";
      ctx.clip(); //裁剪上面的圆形
      ctx.drawImage(list[2].tempFilePath, 46, 856, 100, 100);
      ctx.draw(false, function () {
        _index2.default.canvasToTempFilePath({
          canvasId: canvasName, success: function success(_ref) {
            var tempFilePath = _ref.tempFilePath;

            resolve(tempFilePath);
          }
        });
      });
    });
  });
};

// 删除本地缓存文件
var removeLocalFile = exports.removeLocalFile = function removeLocalFile(callback) {
  _index2.default.getSavedFileList().then(function (res) {
    if (res.fileList.length > 0) {
      res.fileList.map(function (file) {
        _index2.default.removeSavedFile({ filePath: file.filePath });
      });
      callback && callback();
    } else {
      callback && callback();
    }
  });
};

// 把当前画布指定区域的内容导出生成指定大小的图片
var canvasToTempFilePath = exports.canvasToTempFilePath = function canvasToTempFilePath(canvasName, callback) {
  _index2.default.canvasToTempFilePath({
    canvasId: canvasName,
    success: function success(data) {
      var tempFilePath = data.tempFilePath;
      callback(tempFilePath);
    },
    fail: function fail(res) {
      console.log(res);
    }
  });
};
// 保存文件到本地
var saveFile = exports.saveFile = function saveFile(tempFilePath, callback) {
  _index2.default.saveFile({
    tempFilePath: tempFilePath,
    success: function success(res1) {
      _index2.default.hideLoading();
      console.log('成功' + res1.savedFilePath);
      callback(res1.savedFilePath);
    },

    fail: function fail(res) {
      console.log('失败');
    }
  });
};
// 下载文件
var downloadFile = exports.downloadFile = function downloadFile(imgUrl, callback) {
  _index2.default.downloadFile({
    url: imgUrl,
    success: function success(param) {
      if (param.statusCode === 200) {

        callback(param);
      }
    },
    fail: function fail(res) {
      console.log('生成失败请稍后重试');
    }
  });
};

// 生成名片海报
var makeCardPoster = exports.makeCardPoster = function makeCardPoster(data) {
  return new Promise(function (resolve) {
    var bgImg = data.bgImg,
        qrcodeImg = data.qrcodeImg,
        name = data.name,
        nickname = data.nickname,
        avatarurl = data.avatarurl,
        company = data.company,
        post = data.post,
        canvasName = data.canvasName;

    var ctx = _index2.default.createCanvasContext(canvasName);
    var _qrcodeImg = getDownloadURL(qrcodeImg, 'alioss');
    var _avatarurl = getDownloadURL(avatarurl, 'avatar');

    Promise.all([_index2.default.downloadFile({ url: _avatarurl }), _index2.default.downloadFile({ url: _qrcodeImg })]).then(function (list) {

      // 绘制背景
      ctx.setFillStyle("#ffe200");
      ctx.fillRect(0, 0, 640, 1040);
      ctx.drawImage(bgImg, 0, 0, 640, 1040);

      // 绘制小程序码
      ctx.drawImage(qrcodeImg, 170, 530, 290, 290);

      //绘制名字
      ctx.setFontSize(35);
      ctx.setFillStyle('#000');
      ctx.setTextAlign('center');
      ctx.fillText(name, 320, 356, 450);
      ctx.stroke();

      // 绘制圆角矩形
      roundRect(ctx, 220, 180, 320, 80, 10);
      ctx.strokeStyle = "#7ff724";
      ctx.setFillStyle("#7ff724");
      ctx.fill();
      ctx.stroke();

      //推荐语
      ctx.setFontSize(20);
      ctx.setFillStyle('#000');
      ctx.setTextAlign('left');

      // 文字换行
      var str = nickname + ':给你推荐个人,Ta特别靠谱~';
      var lineWidth = 0;
      var canvasWidth = 260; //文字宽度
      var initHeight = 213; //绘制字体距离canvas顶部初始的高度
      var lastSubStrIndex = 0; //每次开始截取的字符串的索引
      for (var i = 0; i < str.length; i++) {
        lineWidth += ctx.measureText(str[i]).width;
        if (lineWidth > canvasWidth) {
          ctx.fillText(str.substring(lastSubStrIndex, i), 250, initHeight); //绘制截取部分
          initHeight += 30; //20为字体的高度
          lineWidth = 0;
          lastSubStrIndex = i;
        }
        if (i == str.length - 1) {
          //绘制剩余部分
          ctx.fillText(str.substring(lastSubStrIndex, i + 1), 250, initHeight);
        }
      }
      ctx.stroke();
      //绘制职位
      ctx.setFontSize(25);
      ctx.setFillStyle('#606060');
      ctx.setTextAlign('center');
      ctx.fillText(post, 320, 446, 450);
      ctx.stroke();
      //绘制公司
      ctx.setFontSize(25);
      ctx.setFillStyle('#606060');
      ctx.setTextAlign('center');
      ctx.fillText(company, 320, 500, 450);
      ctx.stroke();
      //长按或扫一扫了解Ta
      ctx.setFontSize(20);
      ctx.setFillStyle('#a5a5a5');
      ctx.setTextAlign('center');
      ctx.fillText('长按或扫一扫了解Ta', 320, 860, 450);
      ctx.stroke();
      ctx.save();

      // 绘制头像
      ctx.arc(146, 216, 50, 0, 2 * Math.PI); //画出圆
      ctx.strokeStyle = "#ffffff";
      ctx.clip(); //裁剪上面的圆形
      ctx.drawImage(list[0].tempFilePath, 96, 166, 100, 100);
      ctx.restore();

      // 绘制二维码
      ctx.save();
      ctx.arc(315, 675, 65, 0, 2 * Math.PI); //画出圆
      ctx.strokeStyle = "#ffffff";
      ctx.clip();
      ctx.drawImage(list[1].tempFilePath, 50, 0, 400, 400, 249, 610, 130, 130);
      ctx.draw(false, function () {
        _index2.default.canvasToTempFilePath({
          canvasId: canvasName, success: function success(_ref2) {
            var tempFilePath = _ref2.tempFilePath;

            resolve(tempFilePath);
          }
        });
      });
    });
  });
};

// 圆角矩形
var roundRect = exports.roundRect = function roundRect(ctx, x, y, w, h, r) {
  if (w < 2 * r) r = w / 2;
  if (h < 2 * r) r = h / 2;
  ctx.beginPath();
  ctx.moveTo(x + r, y);
  ctx.arcTo(x + w, y, x + w, y + h, r);
  ctx.arcTo(x + w, y + h, x, y + h, r);
  ctx.arcTo(x, y + h, x, y, r);
  ctx.arcTo(x, y, x + w, y, r);
  ctx.closePath();
};

var makeCardShareImg = exports.makeCardShareImg = function makeCardShareImg(data) {
  return new Promise(function (resolve) {
    var bgImg = data.bgImg,
        post = data.post,
        name = data.name,
        phoneImg = data.phoneImg,
        phone = data.phone,
        picture = data.picture,
        addrImg = data.addrImg,
        company = data.company,
        canvasName = data.canvasName;

    var ctx = _index2.default.createCanvasContext(canvasName);
    var _picture = getDownloadURL(picture, 'alioss');
    Promise.all([_index2.default.downloadFile({ url: _picture })]).then(function (list) {

      ctx.setFillStyle("#ffffff");
      ctx.fillRect(0, 0, 750, 400);

      // 背景海报
      ctx.drawImage(bgImg, 0, 0, 750, 600);

      //职位
      ctx.setFontSize(28);
      ctx.fillText(post, 50, 90);

      //姓名
      ctx.setFontSize(60);
      ctx.fillText(name, 50, 170);

      ctx.setFontSize(30);

      // 导航图片
      ctx.drawImage(addrImg, 50, 400, 28, 28);
      //公司
      ctx.fillText(company, 100, 425);

      // 手机图片
      ctx.drawImage(phoneImg, 48, 450, 36, 36);
      //手机号码
      ctx.fillText('TEL: ' + phone, 100, 478);

      //绘制头像
      ctx.arc(550, 200, 100, 0, 2 * Math.PI); //画出圆
      ctx.strokeStyle = "#ffffff";
      ctx.stroke();

      ctx.clip(); //裁剪上面的圆形
      ctx.drawImage(list[0].tempFilePath, 450, 100, 200, 200, 400, 100);

      ctx.draw(false, function () {
        _index2.default.canvasToTempFilePath({ canvasId: canvasName, quality: 1 }).then(function (_ref3) {
          var tempFilePath = _ref3.tempFilePath;

          resolve(tempFilePath);
        });
      });
    });
  });
};

var isObject = exports.isObject = function isObject(value) {
  var type = typeof value === 'undefined' ? 'undefined' : _typeof(value);
  return value != null && type === 'object';
};

//使用递归的方式实现数组、对象的深拷贝
var deepClone = exports.deepClone = function deepClone(obj) {
  if (!isObject(obj)) {
    throw new Error('obj 不是一个对象！');
  }
  var isArray = Array.isArray(obj);
  var cloneObj = isArray ? [] : {};
  for (var key in obj) {
    cloneObj[key] = isObject(obj[key]) ? deepClone(obj[key]) : obj[key];
  }

  return cloneObj;
};

var validateAuth = exports.validateAuth = function validateAuth(name, callback) {
  _index2.default.getSetting().then(function (res) {
    var hasAuth = res.authSetting[name];
    if (hasAuth) {
      callback && callback(true);
    } else {
      callback && callback(false);
    }
  });
};

/**
 * 检测 `phoneNumber` 是否是中国的手机号码。
 *
 * @param phoneNumber 要检测的电话号码
 * @param strict 是否严格模式
 * @returns `phoneNumber` 是中国的手机号码返回 `true`，否则返回 `false`
 */
var isChineseMobilePhoneNumber = exports.isChineseMobilePhoneNumber = function isChineseMobilePhoneNumber(phoneNumber, strict) {
  if (strict === undefined) {
    strict = false;
  }
  return (strict ? /^1(?:3[0-9]|4[5-9]|5[0-9]|6[12456]|7[0-8]|8[0-9]|9[0-9])[0-9]{8}$/ : /^1[3-9][0-9]{9}$/).test(phoneNumber.toString());
};

/**
 * 检查 `value` 是否是一个邮件地址。
 *
 * @param value 要检查的值
 * @returns `value` 是邮件地址返回 `true`，否则返回 `false`
 * @see http://emailregex.com/
 */
var isEmail = exports.isEmail = function isEmail(value) {
  var re = /^(([^<>()[\]\\.,;:\s@"]+(\.[^<>()[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
  return re.test(value);
};